@@ -1,8 +1,11 @@
+import asyncio
 import uuid
 
 import structlog
 from django.core.exceptions import PermissionDenied
 from django.http import Http404
+from django.utils.decorators import sync_and_async_middleware
+from asgiref import sync
 
 from .. import signals
 
@@ -16,49 +19,12 @@ def get_request_header(request, header_key, meta_key):
     return request.META.get(meta_key)
 
 
-class RequestMiddleware:
-    """``RequestMiddleware`` adds request metadata to ``structlog``'s logger context automatically.
-
-    >>> MIDDLEWARE = [
-    ...     # ...
-    ...     'django_structlog.middlewares.RequestMiddleware',
-    ... ]
-
-    """
-
+class BaseRequestMiddleWare:
     def __init__(self, get_response):
         self.get_response = get_response
         self._raised_exception = False
 
-    def __call__(self, request):
-        from ipware import get_client_ip
-
-        request_id = get_request_header(
-            request, "x-request-id", "HTTP_X_REQUEST_ID"
-        ) or str(uuid.uuid4())
-
-        correlation_id = get_request_header(
-            request, "x-correlation-id", "HTTP_X_CORRELATION_ID"
-        )
-
-        structlog.contextvars.bind_contextvars(request_id=request_id)
-        self.bind_user_id(request)
-        if correlation_id:
-            structlog.contextvars.bind_contextvars(correlation_id=correlation_id)
-
-        ip, _ = get_client_ip(request)
-        structlog.contextvars.bind_contextvars(ip=ip)
-        signals.bind_extra_request_metadata.send(
-            sender=self.__class__, request=request, logger=logger
-        )
-
-        logger.info(
-            "request_started",
-            request=self.format_request(request),
-            user_agent=request.META.get("HTTP_USER_AGENT"),
-        )
-        self._raised_exception = False
-        response = self.get_response(request)
+    def handle_response(self, request, response):
         if not self._raised_exception:
             self.bind_user_id(request)
             signals.bind_extra_request_finished_metadata.send(
@@ -72,9 +38,32 @@ class RequestMiddleware:
                 code=response.status_code,
                 request=self.format_request(request),
             )
-
         structlog.contextvars.clear_contextvars()
-        return response
+
+    def prepare(self, request):
+        from ipware import get_client_ip
+
+        request_id = get_request_header(
+            request, "x-request-id", "HTTP_X_REQUEST_ID"
+        ) or str(uuid.uuid4())
+        correlation_id = get_request_header(
+            request, "x-correlation-id", "HTTP_X_CORRELATION_ID"
+        )
+        structlog.contextvars.bind_contextvars(request_id=request_id)
+        self.bind_user_id(request)
+        if correlation_id:
+            structlog.contextvars.bind_contextvars(correlation_id=correlation_id)
+        ip, _ = get_client_ip(request)
+        structlog.contextvars.bind_contextvars(ip=ip)
+        signals.bind_extra_request_metadata.send(
+            sender=self.__class__, request=request, logger=logger
+        )
+        logger.info(
+            "request_started",
+            request=self.format_request(request),
+            user_agent=request.META.get("HTTP_USER_AGENT"),
+        )
+        self._raised_exception = False
 
     @staticmethod
     def format_request(request):
@@ -111,3 +100,37 @@ class RequestMiddleware:
                 if isinstance(user_id, uuid.UUID):
                     user_id = str(user_id)
             structlog.contextvars.bind_contextvars(user_id=user_id)
+
+
+class SyncRequestMiddleware(BaseRequestMiddleWare):
+    def __call__(self, request):
+        self.prepare(request)
+        response = self.get_response(request)
+        self.handle_response(request, response)
+        return response
+
+
+class AsyncRequestMiddleware(BaseRequestMiddleWare):
+    async def __call__(self, request):
+        await sync.sync_to_async(self.prepare)(request)
+        response = await self.get_response(request)
+        await sync.sync_to_async(self.handle_response)(request, response)
+        return response
+
+
+# noinspection PyPep8Naming
+@sync_and_async_middleware
+def RequestMiddleware(get_response):
+    """``RequestMiddleware`` adds request metadata to ``structlog``'s logger context automatically.
+
+    >>> MIDDLEWARE = [
+    ...     # ...
+    ...     'django_structlog.middlewares.RequestMiddleware',
+    ... ]
+
+    """
+
+    # One-time configuration and initialization goes here.
+    if asyncio.iscoroutinefunction(get_response):
+        return AsyncRequestMiddleware(get_response)
+    return SyncRequestMiddleware(get_response)
